import {
  select,
  json,
  scaleBand,
  scaleLinear,
  scaleOrdinal,
  range,
  ascending,
  schemeCategory10,
  selectAll,
  event,
  zoom,
  scaleSqrt
} from 'd3';

const margin = {
    top: 285,
    right: 0,
    bottom: 0,
    left: 285
    },
    width = 700,
    height = 700;

const fontSize = d3.scaleSqrt().range([30, 15]);

const svg = d3.select("graph").append('svg')
    .attr('width', width + margin.left + margin.right)
    .attr('height', height + margin.top + margin.bottom)
		.append("g")
		.attr("transform", "translate(" + margin.left + "," + margin.top + ")");

svg.append("rect")
    .attr("class", "background")
    .attr("width", width)
    .attr("height", height);

d3.json('data.json').then(data => {
    var matrix = [];
    var nodes = data.nodes;
    var total_items = nodes.length;

    var matrixScale = d3.scaleBand().range([0, width]).domain(d3.range(total_items));
    var opacityScale = d3.scaleLinear().domain([0, 10]).range([0.3, 1.0]).clamp(true);
    var colorScale = d3.scaleOrdinal(d3.schemeCategory10);
    
    // Create rows for the matrix
    nodes.forEach(function(node) {
        node.count = 0;
        node.group = stringToInt(node.group);
        node.stars = stringToFloat(node.stars);
        node.sentiment = stringToFloat(node.sentiment);
        matrix[stringToInt(node.index)] = d3.range(total_items).map(item_index => {
            return {
                x: item_index,
                y: stringToInt(node.index),
                z: 0
            };
        });
    });

    // Fill matrix with data from links and count how many times each item appears
    data.links.forEach(function(link) {
        matrix[stringToInt(link.source)][stringToInt(link.target)].z = stringToFloat(link.value);
        matrix[stringToInt(link.target)][stringToInt(link.source)].z = stringToFloat(link.value);
        nodes[stringToInt(link.source)].count += stringToFloat(link.value);
        nodes[stringToInt(link.target)].count += stringToFloat(link.value);
    });

    // Draw each row (translating the y coordinate)
    var rows = svg.selectAll(".row")
        .data(matrix)
        .enter().append("g")
        .attr("class", "row")
        .attr("transform", (d, i) => {
            return "translate(0," + matrixScale(i) + ")";
        });

    var squares = rows.selectAll(".cell")
        .data(d => d.filter(item => item.z > 0))
        .enter().append("rect")
        .attr("class", "cell")
        .attr("x", d => matrixScale(d.x))
        .attr("width", matrixScale.bandwidth())
        .attr("height", matrixScale.bandwidth())
    		.attr("fill", d => colorScale(nodes[d.x].group))
    		.attr("opacity", d => d.z)
        .on("mouseover", mouseover)
        .on("mouseout", mouseout);

    var columns = svg.selectAll(".column")
        .data(matrix)
        .enter().append("g")
        .attr("class", "column")
        .attr("transform", (d, i) => {
            return "translate(" + matrixScale(i) + ")rotate(-90)";
        });

    rows.append("text")
        .attr("class", "label")
        .attr("x", -5)
        .attr("y", matrixScale.bandwidth() / 2)
        .attr("dy", ".32em")
        .attr("text-anchor", "end")
        .text((d, i) => capitalize_Words(nodes[i].name));

    columns.append("text")
        .attr("class", "label")
        .attr("y", 100)
        .attr("y", matrixScale.bandwidth() / 2)
        .attr("dy", ".32em")
        .attr("text-anchor", "start")
        .text((d, i) => capitalize_Words(nodes[i].name));

    // Precompute the orders.
    var orders = {
        name: d3.range(total_items).sort((a, b) => {
            return d3.ascending(nodes[a].name, nodes[b].name);
        }),
        count: d3.range(total_items).sort((a, b) => {
            return nodes[b].count - nodes[a].count;
        }),
        stars: d3.range(total_items).sort((a, b) => {
            return nodes[b].stars - nodes[a].stars;
        }),
        sentiment: d3.range(total_items).sort((a, b) => {
            return nodes[b].sentiment - nodes[a].sentiment;
        }),
        group: d3.range(total_items).sort((a, b) => {
            return d3.ascending(nodes[a].group, nodes[b].group);
        })
    };

    d3.select("#order").on("change", function() {
        changeOrder(this.value);
    });

    function changeOrder(value) {
        matrixScale.domain(orders[value]);
        var t = svg.transition().duration(2000);

        t.selectAll(".row")
            .delay((d, i) => matrixScale(i) * 4)
            .attr("transform", function(d, i) {
                return "translate(0," + matrixScale(i) + ")";
            })
            .selectAll(".cell")
            .delay(d => matrixScale(d.x) * 4)
            .attr("x", d => matrixScale(d.x));

        t.selectAll(".column")
            .delay((d, i) => matrixScale(i) * 4)
            .attr("transform", (d, i) => "translate(" + matrixScale(i) + ")rotate(-90)");
    }

    rows.append("line")
        .attr("x2", width);

    columns.append("line")
        .attr("x1", -width);

    var tooltip = d3.select("body")
        .append("div")
        .attr("class", "tooltip")
        .style("opacity", 0);

    function mouseover(p) {
        d3.selectAll(".row text").classed("active", (d, i) => {
            return i == p.y;
        });
        d3.selectAll(".column text").classed("active", (d, i) => {
            return i == p.x;
        });
        tooltip.transition().duration(200).style("opacity", .9);
        tooltip.html(capitalize_Words(nodes[p.y].name) + " [" + numberToString(nodes[p.y].stars) + 
                     											"] [" + numberToString(nodes[p.y].sentiment) + "]</br>" +
                capitalize_Words(nodes[p.x].name) + " [" + numberToString(nodes[p.x].stars) + 
                     											"] [" + numberToString(nodes[p.x].sentiment) + "]</br>" +
                p.z + " similarity")
            .style("left", (d3.event.pageX + 30) + "px")
            .style("top", (d3.event.pageY - 50) + "px");
    }

    function mouseout() {
        d3.selectAll("text").classed("active", false);
        tooltip.transition().duration(500).style("opacity", 0);
    }
});

/* utils */ 

var stringToInt = function(in_string) {
      return parseInt(in_string);
};

var stringToFloat = function(in_string) {
      return parseFloat(in_string);
};

var numberToString = function(in_number) {
		  return in_number.toString()
};

function capitalize_Words(str){
  return str.replace(/\w\S*/g, function(txt){return txt.charAt(0).toUpperCase() + txt.substr(1).toLowerCase();});
};
